import argparse
import json
import os
import re
import sys
from pathlib import Path
from traceback import print_exception

from auto_dlp import config_intepreter

_wait_after_finishing_execution = True


def manage_music(path: Path, redownload_songs=(), test_names=()):
    original_cur_dir = Path(os.curdir).resolve()
    os.chdir(path)

    try:
        config_file = path / "musicmanager.json"
        if not config_file.exists():
            print(f"No musicmanager.json file found in directory {path.resolve()}", file=sys.stderr)
            return

        with open(config_file) as file:
            string = re.sub("//.*", "", file.read())
            json_data = json.loads(string)

        config_intepreter.execute(json_data, redownload_songs=redownload_songs, test_names=test_names)
    except KeyboardInterrupt:
        print("Program interrupted by user", file=sys.stderr)

    os.chdir(original_cur_dir)

def command_entry_point():
    try:
        main()
    except Exception as e:
        print_exception(e)
        if _wait_after_finishing_execution:
            input()

def main():
    parser = argparse.ArgumentParser(prog="auto-dlp",
                                     description="A light-weight program for managing (and downloading) music",
                                     allow_abbrev=True, add_help=True, exit_on_error=True)
    parser.add_argument("FOLDER",
                        help="The folder in which to execute this program. It must contain a musicmanager.json file.")
    parser.add_argument("-rd", "--redownload", metavar="SONG",
                        help="Will delete the current version of the specified song, meaning that it will be redownloaded on the next execution of this program")
    parser.add_argument("-tn", "--test-name", metavar="NAME",
                        help="Allows to try out what the renaming system does to a specific string")
    parser.add_argument("-w", "--wait", action="store_true", help="After finishing execution, wait for any input")

    args = parser.parse_args()
    global wait_after_finish
    wait_after_finish = args.wait

    exec_dir = Path(args.FOLDER).expanduser().resolve()
    print(f"Executing Music Manger in directory {exec_dir}")

    redownload = () if args.redownload is None else [args.redownload]
    test_names = () if args.test_name is None else [args.test_name]
    manage_music(exec_dir, redownload_songs=redownload, test_names=test_names)

    if args.wait:
        input("Press ENTER to close program")



import os.path as path
import sys
from json import dumps

from requests import get

import auto_dlp.file_locations as fs
import auto_dlp.utils as utils


def _ask_for_api_key():
    key = input("Please enter a valid Youtube API key: ").strip()
    if key == "":
        return _ask_for_api_key()

    fs.touch_file(fs.yt_api_key_file())
    with fs.yt_api_key_file().open("w") as file:
        print(key.strip(), file=file)

    return key


@utils.lazy
def API_KEY():
    if not path.exists(fs.yt_api_key_file()):
        print("Api key file was not found", file=sys.stderr, flush=True)
        return _ask_for_api_key()

    with open(fs.yt_api_key_file()) as file:
        return file.read().strip()


def request(url, params):
    params["key"] = API_KEY()
    response = get(url, params)

    if response.status_code == 200 and "error" not in response.json():
        return response

    try:
        err_msg = response.json()["error"]["message"]
    except KeyError:
        err_msg = "Error message could not be read"

    raise RuntimeError(f"Network error: {response.status_code} {err_msg}")


def get_playlist_items(config, playlist_id):
    def send_request(params):
        params["playlist_id"] = playlist_id
        params["part"] = "contentDetails, snippet"
        params["maxResults"] = 50
        response = request("https://www.googleapis.com/youtube/v3/playlistItems", params)
        items = {}

        try:
            items_json = response.json()["items"]

            for item in items_json:
                pos = item["snippet"]["position"]
                items[pos] = {
                    "name": item["snippet"]["title"],
                    "id": item["contentDetails"]["videoId"]
                }
        except KeyError:
            print(dumps(response.json(), indent=4), file=sys.stderr)
            raise RuntimeError("Failed getting playlist items")

        if "nextPageToken" in response.json():
            return response.json()["pageInfo"]["totalResults"], response.json()["nextPageToken"], items
        return response.json()["pageInfo"]["totalResults"], None, items

    received_items = 0
    result_count, page_token, items_map = send_request({})

    items_list = [None] * result_count

    def place_items():
        for index, item in items_map.items():
            if items_list[index] is None:
                items_list[index] = item
                continue

            raise RuntimeError("The Youtube Api returned the same result twice")

    place_items()
    received_items += 50

    while received_items < result_count:
        _, page_token, items_map = send_request({"pageToken": page_token})
        try:
            place_items()
        except RuntimeError as e:
            if playlist_id in config.fragile_playlists:
                return list(filter(lambda x: x is not None, items_list))
            raise e
        received_items += 50

    return items_list


def get_song_channel(song_id):
    response = request("https://www.googleapis.com/youtube/v3/videos", {
        "key": API_KEY(),
        "part": "snippet",
        "id": song_id
    })

    try:
        channel = response.json()["items"][0]["snippet"]["channelId"]
    except KeyError:
        print(dumps(response.json(), indent=4), file=sys.stderr)
        raise RuntimeError("Failed getting song thumbnails")

    return channel


def get_song_thumbnails(song_id):
    response = request("https://www.googleapis.com/youtube/v3/videos", {
        "key": API_KEY(),
        "part": "snippet",
        "id": song_id
    })

    try:
        thumbnails = response.json()["items"][0]["snippet"]["thumbnails"]
    except KeyError:
        print(dumps(response.json(), indent=4), file=sys.stderr)
        raise RuntimeError("Failed getting song thumbnails")

    return thumbnails


def get_playlist_thumbnails(playlist_id):
    response = request("https://www.googleapis.com/youtube/v3/playlists", {
        "key": API_KEY(),
        "part": "snippet",
        "id": playlist_id
    })

    try:
        thumbnails = response.json()["items"][0]["snippet"]["thumbnails"]
    except KeyError:
        print(dumps(response.json(), indent=4), file=sys.stderr)
        raise RuntimeError("Failed getting song thumbnails")

    return thumbnails


def get_channel_thumbnails(channel_id):
    response = request("https://www.googleapis.com/youtube/v3/channels", {
        "key": API_KEY(),
        "part": "snippet",
        "id": channel_id
    })

    try:
        thumbnails = response.json()["items"][0]["snippet"]["thumbnails"]
    except KeyError:
        print(dumps(response.json(), indent=4), file=sys.stderr)
        raise RuntimeError("Failed getting song thumbnails")

    return thumbnails
